# coding: utf-8

import os
import re

from django import forms
from django.utils.translation import ugettext_lazy as _

from filebrowser.settings import FOLDER_REGEX
from filebrowser.utils import convert_filename


ALNUM_NAME_RE = re.compile(FOLDER_REGEX, re.U)

TRANSPOSE_CHOICES = (
    ("", u"-----"),
    ("0", _(u"Flip horizontal")),
    ("1", _(u"Flip vertical")),
    ("2", _(u"Rotate 90° CW")),
    ("4", _(u"Rotate 90° CCW")),
    ("3", _(u"Rotate 180°")),
)


class CreateDirForm(forms.Form):
    """
    Form for creating a folder.
    """

    name = forms.CharField(widget=forms.TextInput(attrs=dict({'class': 'vTextField'}, max_length=50, min_length=3)), label=_(u'Name'), help_text=_(u'Only letters, numbers, underscores, spaces and hyphens are allowed.'), required=True)

    def __init__(self, path, *args, **kwargs):
        self.path = path
        self.site = kwargs.pop("filebrowser_site", None)
        super(CreateDirForm, self).__init__(*args, **kwargs)

    def clean_name(self):
        "validate name"
        if self.cleaned_data['name']:
            # only letters, numbers, underscores, spaces and hyphens are allowed.
            if not ALNUM_NAME_RE.search(self.cleaned_data['name']):
                raise forms.ValidationError(_(u'Only letters, numbers, underscores, spaces and hyphens are allowed.'))
            # Folder must not already exist.
            if self.site.storage.isdir(os.path.join(self.path, convert_filename(self.cleaned_data['name']))):
                raise forms.ValidationError(_(u'The Folder already exists.'))
        return convert_filename(self.cleaned_data['name'])


class ChangeForm(forms.Form):
    """
    Form for renaming a file/folder.
    """

    custom_action = forms.ChoiceField(label=_(u'Actions'), required=False)
    name = forms.CharField(widget=forms.TextInput(attrs=dict({'class': 'vTextField'}, max_length=50, min_length=3)), label=_(u'Name'), help_text=_(u'Only letters, numbers, underscores, spaces and hyphens are allowed.'), required=True)

    def __init__(self, *args, **kwargs):
        self.path = kwargs.pop("path", None)
        self.fileobject = kwargs.pop("fileobject", None)
        self.site = kwargs.pop("filebrowser_site", None)
        super(ChangeForm, self).__init__(*args, **kwargs)

        # Initialize choices of custom action
        choices = [("", u"-----")]
        for name, action in self.site.applicable_actions(self.fileobject):
            choices.append((name, action.short_description))
        self.fields['custom_action'].choices = choices

    def clean_name(self):
        "validate name"
        if self.cleaned_data['name']:
            # only letters, numbers, underscores, spaces and hyphens are allowed.
            if not ALNUM_NAME_RE.search(self.cleaned_data['name']):
                raise forms.ValidationError(_(u'Only letters, numbers, underscores, spaces and hyphens are allowed.'))
            #  folder/file must not already exist.
            if self.site.storage.isdir(os.path.join(self.path, convert_filename(self.cleaned_data['name']))) and os.path.join(self.path, convert_filename(self.cleaned_data['name'])) != self.fileobject.path:
                raise forms.ValidationError(_(u'The Folder already exists.'))
            elif self.site.storage.isfile(os.path.join(self.path, convert_filename(self.cleaned_data['name']))) and os.path.join(self.path, convert_filename(self.cleaned_data['name'])) != self.fileobject.path:
                raise forms.ValidationError(_(u'The File already exists.'))
        return convert_filename(self.cleaned_data['name'])



# coding: utf-8

import os

from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured
from django.urls import reverse
from django.http import HttpResponseRedirect
from django.utils.encoding import smart_text
from django.utils.translation import gettext as _

from filebrowser.templatetags.fb_tags import query_helper


def get_path(path, site):
    converted_path = smart_text(os.path.join(site.directory, path))
    if not path.startswith('.') and not os.path.isabs(converted_path):
        if site.storage.isdir(converted_path):
            return path


def get_file(path, filename, site):
    # Files and directories are valid
    converted_path = smart_text(os.path.join(site.directory, path, filename))
    if not path.startswith('.') and not filename.startswith('.') and not os.path.isabs(converted_path):
        if site.storage.isfile(converted_path) or site.storage.isdir(converted_path):
            return filename


def path_exists(site, function):
    "Check if the given path exists."

    def decorator(request, *args, **kwargs):
        # TODO: This check should be moved to a better location than a decorator
        if get_path('', site=site) is None:
            # The storage location does not exist, raise an error to prevent eternal redirecting.
            raise ImproperlyConfigured(_("Error finding Upload-Folder (site.storage.location + site.directory). Maybe it does not exist?"))
        if get_path(request.GET.get('dir', ''), site=site) is None:
            msg = _('The requested Folder does not exist.')
            messages.add_message(request, messages.ERROR, msg)
            redirect_url = reverse("filebrowser:fb_browse", current_app=site.name) + query_helper(request.GET, u"", "dir")
            return HttpResponseRedirect(redirect_url)
        return function(request, *args, **kwargs)
    return decorator


def file_exists(site, function):
    "Check if the given file exists."

    def decorator(request, *args, **kwargs):
        file_path = get_file(request.GET.get('dir', ''), request.GET.get('filename', ''), site=site)
        if file_path is None:
            msg = _('The requested File does not exist.')
            messages.add_message(request, messages.ERROR, msg)
            redirect_url = reverse("filebrowser:fb_browse", current_app=site.name) + query_helper(request.GET, u"", "dir")
            return HttpResponseRedirect(redirect_url)
        return function(request, *args, **kwargs)
    return decorator



# coding: utf-8

# FIXME
# These tests are designed to run against production, not testing. They
# would be better suited for the Django Check Framework.
#
# https://docs.djangoproject.com/en/1.8/topics/checks/


# # PYTHON IMPORTS
# import os

# # DJANGO IMPORTS
# from django.test import TestCase
# from django.conf import settings

# # FILEBROWSER IMPORTS
# from filebrowser.settings import *


# class SettingsTests(TestCase):

#     def setUp(self):
#         pass

#     def test_directory(self):
#         """
#         Test that ``MEDIA_ROOT`` plus ``DIRECTORY`` exists.
#         """
#         self.assertEqual(os.path.exists(os.path.join(settings.MEDIA_ROOT, DIRECTORY)), 1)
#         # Check for trailing slash
#         self.assertEqual(os.path.basename(DIRECTORY), '')

#     def test_versions_basedir(self):
#         """
#         Test that ``MEDIA_ROOT`` plus ``VERSIONS_BASEDIR`` exists.
#         """
#         self.assertEqual(os.path.exists(os.path.join(settings.MEDIA_ROOT, VERSIONS_BASEDIR)), 1)

#     def test_admin_thumbnail(self):
#         """
#         Test if ``ADMIN_THUMBNAIL`` is set and is part of ``VERSIONS``.
#         """
#         self.assertNotEqual(ADMIN_THUMBNAIL, '')
#         self.assertIn(ADMIN_THUMBNAIL, VERSIONS)

#     def test_admin_versions(self):
#         """
#         Test if ``ADMIN_VERSIONS`` are part of ``VERSIONS``.
#         """
#         for item in ADMIN_VERSIONS:
#             self.assertIn(item, VERSIONS)

#     def test_placeholder(self):
#         """
#         Test if ``PLACEHOLDER`` exists.
#         """
#         self.assertEqual(os.path.exists(os.path.join(settings.MEDIA_ROOT, PLACEHOLDER)), 1)

#     def test_show_placeholder(self):
#         """
#         Test if ``SHOW_PLACEHOLDER`` is in ``True, False``.
#         """
#         self.assertIn(SHOW_PLACEHOLDER, [True, False])

#     def test_force_placeholder(self):
#         """
#         Test if ``FORCE_PLACEHOLDER`` is in ``True, False``.
#         """
#         self.assertIn(FORCE_PLACEHOLDER, [True, False])

#     def test_strict_pil(self):
#         """
#         Test if ``STRICT_PIL`` is in ``True, False``.
#         """
#         self.assertIn(STRICT_PIL, [True, False])

#     def test_normalize_filename(self):
#         """
#         Test if ``NORMALIZE_FILENAME`` is in ``True, False``.
#         """
#         self.assertIn(NORMALIZE_FILENAME, [True, False])

#     def test_convert_filename(self):
#         """
#         Test if ``CONVERT_FILENAME`` is in ``True, False``.
#         """
#         self.assertIn(CONVERT_FILENAME, [True, False])

#     def test_default_sorting_by(self):
#         """
#         Test if ``DEFAULT_SORTING_BY`` is in ``date, filesize, filename_lower, filetype_checked``.
#         """
#         self.assertIn(DEFAULT_SORTING_BY, ['date', 'filesize', 'filename_lower', 'filetype_checked'])

#     def test_default_sorting_order(self):
#         """
#         Test if ``DEFAULT_SORTING_ORDER`` is in ``asc, desc``.
#         """
#         self.assertIn(DEFAULT_SORTING_ORDER, ['asc', 'desc'])

#     def test_search_traverse(self):
#         """
#         Test if ``SEARCH_TRAVERSE`` is in ``True, False``.
#         """
#         self.assertIn(SEARCH_TRAVERSE, [True, False])

#     def test_overwrite_existing(self):
#         """
#         Test if ``OVERWRITE_EXISTING`` is in ``True, False``.
#         """
#         self.assertIn(OVERWRITE_EXISTING, [True, False])



# coding: utf-8
import os
import shutil

from django.conf import settings
from django.template import Context, Template, TemplateSyntaxError
from unittest.mock import patch

from tests.base import FilebrowserTestCase as TestCase
from filebrowser.settings import STRICT_PIL
from filebrowser import utils
from filebrowser.utils import scale_and_crop, process_image

if STRICT_PIL:
    from PIL import Image
else:
    try:
        from PIL import Image
    except ImportError:
        import Image


def processor_mark_1(im, **kwargs):
    im.mark_1 = True
    return im


def processor_mark_2(im, **kwargs):
    im.mark_2 = True
    return im


class ImageProcessorsTests(TestCase):
    def setUp(self):
        super(ImageProcessorsTests, self).setUp()
        shutil.copy(self.STATIC_IMG_PATH, self.FOLDER_PATH)

        utils._default_processors = None
        self.im = Image.open(self.F_IMAGE.path_full)

    def tearDown(self):
        super(ImageProcessorsTests, self).tearDown()
        utils._default_processors = None

    def test_process_image_calls_scale_and_crop(self):
        version = process_image(self.im, {'width': 500, 'height': ""})
        self.assertEqual(version.size, (500, 375))

    @patch('filebrowser.utils.VERSION_PROCESSORS', [
        'tests.test_versions.processor_mark_1',
        'tests.test_versions.processor_mark_2',
    ])
    def test_process_image_calls_the_stack_of_processors_in_settings(self):
        version = process_image(self.im, {})
        self.assertTrue(hasattr(version, 'mark_1'))
        self.assertTrue(hasattr(version, 'mark_2'))

    @patch('filebrowser.utils.VERSION_PROCESSORS', [
        'tests.test_versions.processor_mark_1',
    ])
    def test_process_image_calls_only_explicit_provided_processors(self):
        version = process_image(self.im, {}, processors=[processor_mark_2])
        self.assertFalse(hasattr(version, 'mark_1'))
        self.assertTrue(hasattr(version, 'mark_2'))


class ScaleAndCropTests(TestCase):
    def setUp(self):
        super(ScaleAndCropTests, self).setUp()
        shutil.copy(self.STATIC_IMG_PATH, self.FOLDER_PATH)

        self.im = Image.open(self.F_IMAGE.path_full)

    def test_do_not_scale(self):
        version = scale_and_crop(self.im, "", "", "")
        self.assertEqual(version.size, self.im.size)

    def test_do_not_scale_if_desired_size_is_equal_to_original(self):
        width, height = self.im.size
        version = scale_and_crop(self.im, width, height, "")
        self.assertIs(version, self.im)
        self.assertEqual(version.size, (width, height))

    def test_scale_width(self):
        version = scale_and_crop(self.im, 500, "", "")
        self.assertEqual(version.size, (500, 375))

    def test_scale_height(self):
        # new height 375 > 500/375
        version = scale_and_crop(self.im, "", 375, "")
        self.assertEqual(version.size, (500, 375))

    def test_scale_no_upscale_too_wide(self):
        version = scale_and_crop(self.im, 1500, "", "")
        self.assertIs(version, self.im)

    def test_scale_no_upscale_too_tall(self):
        version = scale_and_crop(self.im, "", 1125, "")
        self.assertIs(version, self.im)

    def test_scale_no_upscale_too_wide_and_tall(self):
        version = scale_and_crop(self.im, 1500, 1125, "")
        self.assertIs(version, self.im)

    def test_scale_with_upscale_width(self):
        version = scale_and_crop(self.im, 1500, "", "upscale")
        self.assertEqual(version.size, (1500, 1125))

    def test_scale_with_upscale_height(self):
        version = scale_and_crop(self.im, "", 1125, "upscale")
        self.assertEqual(version.size, (1500, 1125))

    def test_scale_with_upscale_width_and_height(self):
        version = scale_and_crop(self.im, 1500, 1125, "upscale")
        self.assertEqual(version.size, (1500, 1125))

    def test_scale_with_upscale_width_and_zero_height(self):
        version = scale_and_crop(self.im, 1500, 0, "upscale")
        self.assertEqual(version.size, (1500, 1125))

    def test_scale_with_upscale_zero_width_and_height(self):
        version = scale_and_crop(self.im, 0, 1125, "upscale")
        self.assertEqual(version.size, (1500, 1125))

    def test_scale_with_upscale_width_too_small_for_upscale(self):
        version = scale_and_crop(self.im, 500, "", "upscale")
        self.assertEqual(version.size, (500, 375))

    def test_scale_with_upscale_height_too_small_for_upscale(self):
        version = scale_and_crop(self.im, "", 375, "upscale")
        self.assertEqual(version.size, (500, 375))

    def test_crop_width_and_height(self):
        version = scale_and_crop(self.im, 500, 500, "crop")
        self.assertEqual(version.size, (500, 500))

    def test_crop_width_and_height_too_large_no_upscale(self):
        # new width 1500 and height 1500 w. crop > false (upscale missing)
        version = scale_and_crop(self.im, 1500, 1500, "crop")
        self.assertIs(version, self.im)

    def test_crop_width_and_height_too_large_with_upscale(self):
        version = scale_and_crop(self.im, 1500, 1500, "crop,upscale")
        self.assertEqual(version.size, (1500, 1500))

    def test_width_smaller_but_height_bigger_no_upscale(self):
        # new width 500 and height 1125
        # new width is smaller than original, but new height is bigger
        # width has higher priority
        version = scale_and_crop(self.im, 500, 1125, "")
        self.assertEqual(version.size, (500, 375))

    def test_width_smaller_but_height_bigger_with_upscale(self):
        # same with upscale
        version = scale_and_crop(self.im, 500, 1125, "upscale")
        self.assertEqual(version.size, (500, 375))

    def test_width_bigger_but_height_smaller_no_upscale(self):
        # new width 1500 and height 375
        # new width is bigger than original, but new height is smaller
        # height has higher priority
        version = scale_and_crop(self.im, 1500, 375, "")
        self.assertEqual(version.size, (500, 375))

    def test_width_bigger_but_height_smaller_with_upscale(self):
        # same with upscale
        version = scale_and_crop(self.im, 1500, 375, "upscale")
        self.assertEqual(version.size, (500, 375))


class VersionTemplateTagTests(TestCase):
    """Test basic version uses

    Eg:
    {% version obj "large" %}
    {% version path "large" %}

    """

    def setUp(self):
        super(VersionTemplateTagTests, self).setUp()
        shutil.copy(self.STATIC_IMG_PATH, self.FOLDER_PATH)

        os.makedirs(self.PLACEHOLDER_PATH)
        shutil.copy(self.STATIC_IMG_PATH, self.PLACEHOLDER_PATH)

    def test_wrong_token(self):
        self.assertRaises(TemplateSyntaxError, lambda: Template('{% load fb_versions %}{% version obj.path %}'))
        self.assertRaises(TemplateSyntaxError, lambda: Template('{% load fb_versions %}{% version %}'))

    def test_invalid_version(self):
        # FIXME: should this throw an error?
        t = Template('{% load fb_versions %}{% version obj "invalid" %}')
        c = Context({"obj": self.F_IMAGE})
        r = t.render(c)
        self.assertEqual(r, "")

    def test_hardcoded_path(self):
        t = Template('{% load fb_versions %}{% version path "large" %}')
        c = Context({"obj": self.F_IMAGE, "path": "_test/uploads/folder/testimage.jpg"})
        r = t.render(c)
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))

    def test_with_obj(self):
        t = Template('{% load fb_versions %}{% version obj "large" %}')
        c = Context({"obj": self.F_IMAGE})
        r = t.render(c)
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))

    def test_with_obj_path(self):
        t = Template('{% load fb_versions %}{% version obj.path "large" %}')
        c = Context({"obj": self.F_IMAGE})
        r = t.render(c)
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))

    @patch.dict('filebrowser.templatetags.fb_versions.VERSIONS', {'fixedheight': {'verbose_name': 'Fixed height', 'width': '', 'height': 100, 'opts': ''}})
    def test_size_fixedheight(self):
        t = Template('{% load fb_versions %}{% version path "fixedheight" %}')
        c = Context({"obj": self.F_IMAGE, "path": "_test/uploads/folder/testimage.jpg"})
        r = t.render(c)
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_fixedheight.jpg"))

    def test_non_existing_path(self):
        # FIXME: templatetag version with non-existing path
        t = Template('{% load fb_versions %}{% version path "large" %}')
        c = Context({"obj": self.F_IMAGE, "path": "_test/uploads/folder/testimagexxx.jpg"})
        r = t.render(c)
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, ""))

    @patch('filebrowser.templatetags.fb_versions.SHOW_PLACEHOLDER', True)
    @patch('filebrowser.templatetags.fb_versions.FORCE_PLACEHOLDER', True)
    def test_force_placeholder_with_existing_image(self, ):
        t = Template('{% load fb_versions %}{% version obj.path suffix %}')
        c = Context({"obj": self.F_IMAGE, "suffix": "large"})
        r = t.render(c)
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))

    @patch('filebrowser.templatetags.fb_versions.SHOW_PLACEHOLDER', True)
    @patch('filebrowser.templatetags.fb_versions.FORCE_PLACEHOLDER', True)
    def test_force_placeholder_without_existing_image(self):
        t = Template('{% load fb_versions %}{% version obj.path suffix %}')
        c = Context({"obj": self.F_MISSING, "suffix": "large"})
        r = t.render(c)
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))

    @patch('filebrowser.templatetags.fb_versions.SHOW_PLACEHOLDER', True)
    def test_no_force_placeholder_with_existing_image(self):
        t = Template('{% load fb_versions %}{% version obj.path suffix %}')
        c = Context({"obj": self.F_IMAGE, "suffix": "large"})
        r = t.render(c)
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))

    @patch('filebrowser.templatetags.fb_versions.SHOW_PLACEHOLDER', True)
    def test_no_force_placeholder_without_existing_image(self):
        t = Template('{% load fb_versions %}{% version obj.path suffix %}')
        c = Context({"obj": self.F_MISSING, "suffix": "large"})
        r = t.render(c)
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))

    # def test_permissions(self):
    # FIXME: Test permissions by creating file AFTER we patch DEFAULT_PERMISSIONS
    #     permissions_file = oct(os.stat(os.path.join(settings.MEDIA_ROOT, "_test/_versions/folder/testimage_large.jpg")).st_mode & 0o777)
    #     self.assertEqual(oct(0o755), permissions_file)


class VersionAsTemplateTagTests(TestCase):
    """Test variable version uses

    Eg:
    {% version obj "large" as version_large %}
    {% version path "large" as version_large %}

    """

    def setUp(self):
        super(VersionAsTemplateTagTests, self).setUp()
        shutil.copy(self.STATIC_IMG_PATH, self.FOLDER_PATH)

        os.makedirs(self.PLACEHOLDER_PATH)
        shutil.copy(self.STATIC_IMG_PATH, self.PLACEHOLDER_PATH)

    def test_hardcoded_path(self):
        t = Template('{% load fb_versions %}{% version path "large" as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE, "path": "_test/uploads/folder/testimage.jpg"})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))

    def test_obj_path(self):
        t = Template('{% load fb_versions %}{% version obj.path "large" as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))

    def test_with_obj(self):
        t = Template('{% load fb_versions %}{% version obj "large" as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))

    def test_with_suffix_as_variable(self):
        t = Template('{% load fb_versions %}{% version obj suffix as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE, "suffix": "large"})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))

    def test_non_existing_path(self):
        # FIXME: templatetag version with non-existing path
        t = Template('{% load fb_versions %}{% version path "large" as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE, "path": "_test/uploads/folder/testimagexxx.jpg"})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, ""))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, ""))

    @patch('filebrowser.templatetags.fb_versions.SHOW_PLACEHOLDER', True)
    @patch('filebrowser.templatetags.fb_versions.FORCE_PLACEHOLDER', True)
    def test_force_placeholder_with_existing_image(self):
        t = Template('{% load fb_versions %}{% version obj suffix as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE, "suffix": "large"})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))

    @patch('filebrowser.templatetags.fb_versions.SHOW_PLACEHOLDER', True)
    def test_no_force_placeholder_with_existing_image(self):
        t = Template('{% load fb_versions %}{% version obj suffix as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE, "suffix": "large"})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))

    @patch('filebrowser.templatetags.fb_versions.SHOW_PLACEHOLDER', True)
    @patch('filebrowser.templatetags.fb_versions.FORCE_PLACEHOLDER', True)
    def test_force_placeholder_with_non_existing_image(self):
        t = Template('{% load fb_versions %}{% version obj suffix as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_MISSING, "suffix": "large"})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))

    @patch('filebrowser.templatetags.fb_versions.SHOW_PLACEHOLDER', True)
    def test_no_force_placeholder_with_non_existing_image(self):
        t = Template('{% load fb_versions %}{% version obj suffix as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_MISSING, "suffix": "large"})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))


class VersionObjectTemplateTagTests(TestCase):
    """Test version uses

    Eg:
    {% version obj "large" as version_large %}
    {% version path "large" as version_large %}

    """
    def setUp(self):
        super(VersionObjectTemplateTagTests, self).setUp()
        shutil.copy(self.STATIC_IMG_PATH, self.FOLDER_PATH)
        shutil.copy(self.STATIC_IMG_BAD_PATH, self.FOLDER_PATH)

        os.makedirs(self.PLACEHOLDER_PATH)
        shutil.copy(self.STATIC_IMG_PATH, self.PLACEHOLDER_PATH)

    def test_wrong_token(self):
        self.assertRaises(TemplateSyntaxError, lambda: Template('{% load fb_versions %}{% version obj.path %}'))
        self.assertRaises(TemplateSyntaxError, lambda: Template('{% load fb_versions %}{% version %}'))
        # next one does not raise an error anymore, because we use version instead of version_object
        # leave here for reference and delete later
        # self.assertRaises(TemplateSyntaxError, lambda: Template('{% load fb_versions %}{% version obj.path "medium" %}'))

    def test_hardcoded_path(self):
        t = Template('{% load fb_versions %}{% version path "large" as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE, "path": "_test/uploads/folder/testimage.jpg"})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))

    def test_obj_path(self):
        t = Template('{% load fb_versions %}{% version obj.path "large" as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))

    def test_with_obj(self):
        t = Template('{% load fb_versions %}{% version obj "large" as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))

    def test_suffix_as_variable(self):
        t = Template('{% load fb_versions %}{% version obj suffix as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE, "suffix": "large"})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))

    def test_non_existing_path(self):
        # FIXME: templatetag version with non-existing path
        t = Template('{% load fb_versions %}{% version path "large" as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE, "path": "_test/uploads/folder/testimagexxx.jpg"})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, ""))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, ""))

    def test_jpg_not_an_image(self):
        t = Template('{% load fb_versions %}{% version obj "large" as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE_BAD})
        r = t.render(c)
        self.assertEqual(c["version_large"], "")
        self.assertEqual(r, "")

    @patch('filebrowser.templatetags.fb_versions.SHOW_PLACEHOLDER', True)
    @patch('filebrowser.templatetags.fb_versions.FORCE_PLACEHOLDER', True)
    def test_force_with_existing_image(self):
        t = Template('{% load fb_versions %}{% version obj suffix as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE, "suffix": "large"})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))

    @patch('filebrowser.templatetags.fb_versions.SHOW_PLACEHOLDER', True)
    def test_no_force_with_existing_image(self):
        t = Template('{% load fb_versions %}{% version obj suffix as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_IMAGE, "suffix": "large"})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/folder/testimage_large.jpg"))

    @patch('filebrowser.templatetags.fb_versions.SHOW_PLACEHOLDER', True)
    @patch('filebrowser.templatetags.fb_versions.FORCE_PLACEHOLDER', True)
    def test_force_with_non_existing_image(self):
        t = Template('{% load fb_versions %}{% version obj suffix as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_MISSING, "suffix": "large"})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))

    @patch('filebrowser.templatetags.fb_versions.SHOW_PLACEHOLDER', True)
    @patch('filebrowser.templatetags.fb_versions.FORCE_PLACEHOLDER', False)
    def test_no_force_with_non_existing_image(self):
        t = Template('{% load fb_versions %}{% version obj suffix as version_large %}{{ version_large.url }}')
        c = Context({"obj": self.F_MISSING, "suffix": "large"})
        r = t.render(c)
        self.assertEqual(c["version_large"].url, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))
        self.assertEqual(r, os.path.join(settings.MEDIA_URL, "_test/_versions/placeholders/testimage_large.jpg"))



import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='django-filebrowser-no-grappelli2',
    version='3.8.3',
    description='Media-Management no Grappelli',
    long_description=read('README.rst'),
    url='https://github.com/christianwgd/django-filebrowser-no-grappelli',
    download_url='',
    author='Patrick Kranzlmueller, Axel Swoboda (vonautomatisch), Maxim Sukharev, Christian Wiegand',
    author_email='christianwgd@gmail.com',
    maintainer='Christian Wiegand',
    maintainer_email='christianwgd@gmail.com',
    license='BSD',
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    zip_safe=False,
)


# coding: utf-8

import shutil

from unittest.mock import patch

from filebrowser.settings import VERSIONS
from tests import FilebrowserTestCase as TestCase

from filebrowser.namers import OptionsNamer


class BaseNamerTests(TestCase):
    NAMER_CLASS = OptionsNamer

    def setUp(self):
        super(BaseNamerTests, self).setUp()
        shutil.copy(self.STATIC_IMG_PATH, self.FOLDER_PATH)

    def _get_namer(self, version_suffix, file_object=None, **extra_options):
        if not file_object:
            file_object = self.F_IMAGE

        extra_options.update(VERSIONS.get(version_suffix, {}))

        return self.NAMER_CLASS(
            file_object=file_object,
            version_suffix=version_suffix,
            filename_root=file_object.filename_root,
            extension=file_object.extension,
            options=extra_options,
        )


class OptionsNamerTests(BaseNamerTests):

    def test_should_return_correct_version_name_using_predefined_versions(self):
        expected = [
            ("admin_thumbnail", "testimage_admin-thumbnail--60x60--opts-crop.jpg", ),
            ("thumbnail", "testimage_thumbnail--60x60--opts-crop.jpg", ),
            ("small", "testimage_small--140x0.jpg", ),
            ("medium", "testimage_medium--300x0.jpg", ),
            ("big", "testimage_big--460x0.jpg", ),
            ("large", "testimage_large--680x0.jpg", ),
        ]

        for version_suffix, expected_name in expected:
            namer = self._get_namer(version_suffix)
            self.assertEqual(namer.get_version_name(), expected_name)

    @patch('filebrowser.namers.VERSION_NAMER', 'filebrowser.namers.OptionsNamer')
    def test_should_return_correct_original_name_using_predefined_versions(self):
        expected = 'testimage.jpg'
        for version_suffix in VERSIONS.keys():
            file_object = self.F_IMAGE.version_generate(version_suffix)
            namer = self._get_namer(version_suffix, file_object)
            self.assertNotEqual(file_object.filename, expected)

            self.assertEqual(namer.get_original_name(), expected)
            self.assertEqual(file_object.original_filename, expected)

    def test_should_append_extra_options(self):
        expected = [
            (
                "thumbnail",
                "testimage_thumbnail--60x60--opts-crop--sepia.jpg",
                {'sepia': True}
            ), (
                "small",
                "testimage_small--140x0--thumb--transparency-08.jpg",
                {'transparency': 0.8, 'thumb': True}
            ), (
                "large",
                "testimage_large--680x0--nested-xpto-ops--thumb.jpg",
                {'thumb': True, 'nested': {'xpto': 'ops'}}
            ),
        ]

        for version_suffix, expected_name, extra_options in expected:
            namer = self._get_namer(version_suffix, **extra_options)
            self.assertEqual(namer.get_version_name(), expected_name)

    def test_generated_version_name_options_list_should_be_ordered(self):
        "the order is important to always generate the same name"
        expected = [
            (
                "small",
                "testimage_small--140x0--a--x-crop--z-4.jpg",
                {'z': 4, 'a': True, 'x': 'crop'}
            ),
        ]

        for version_suffix, expected_name, extra_options in expected:
            namer = self._get_namer(version_suffix, **extra_options)
            self.assertEqual(namer.get_version_name(), expected_name)
